# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
import base64
from typing import Any, Union, Mapping
from typing_extensions import Self, override

import httpx

from . import _exceptions, _legacy_response
from ._qs import Querystring
from ._types import (
    NOT_GIVEN,
    Body,
    Omit,
    Query,
    Headers,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
)
from ._utils import (
    is_given,
    get_async_library,
)
from ._version import __version__
from ._response import to_streamed_response_wrapper, async_to_streamed_response_wrapper
from .resources import (
    events,
    ledgers,
    returns,
    webhooks,
    documents,
    line_items,
    connections,
    paper_items,
    validations,
    bulk_results,
    bulk_requests,
    payment_flows,
    counterparties,
    ledger_entries,
    legal_entities,
    account_details,
    ledger_accounts,
    routing_details,
    virtual_accounts,
    expected_payments,
    external_accounts,
    ledgerable_events,
    payment_references,
    ledger_event_handlers,
    foreign_exchange_quotes,
    account_collection_flows,
    incoming_payment_details,
    connection_legal_entities,
    ledger_account_categories,
    ledger_account_statements,
    legal_entity_associations,
    ledger_account_settlements,
    ledger_account_balance_monitors,
)
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import APIStatusError, ModernTreasuryError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
    make_request_options,
)
from .resources.invoices import invoices
from .types.ping_response import PingResponse
from .resources.transactions import transactions
from .resources.payment_orders import payment_orders
from .resources.internal_accounts import internal_accounts
from .resources.ledger_transactions import ledger_transactions

__all__ = [
    "Timeout",
    "Transport",
    "ProxiesTypes",
    "RequestOptions",
    "ModernTreasury",
    "AsyncModernTreasury",
    "Client",
    "AsyncClient",
]


class ModernTreasury(SyncAPIClient):
    connections: connections.Connections
    counterparties: counterparties.Counterparties
    events: events.Events
    expected_payments: expected_payments.ExpectedPayments
    external_accounts: external_accounts.ExternalAccounts
    incoming_payment_details: incoming_payment_details.IncomingPaymentDetails
    invoices: invoices.Invoices
    documents: documents.Documents
    account_collection_flows: account_collection_flows.AccountCollectionFlows
    account_details: account_details.AccountDetails
    routing_details: routing_details.RoutingDetails
    internal_accounts: internal_accounts.InternalAccounts
    ledgers: ledgers.Ledgers
    ledgerable_events: ledgerable_events.LedgerableEvents
    ledger_account_categories: ledger_account_categories.LedgerAccountCategories
    ledger_accounts: ledger_accounts.LedgerAccounts
    ledger_account_balance_monitors: ledger_account_balance_monitors.LedgerAccountBalanceMonitors
    ledger_account_statements: ledger_account_statements.LedgerAccountStatements
    ledger_entries: ledger_entries.LedgerEntries
    ledger_event_handlers: ledger_event_handlers.LedgerEventHandlers
    ledger_transactions: ledger_transactions.LedgerTransactions
    line_items: line_items.LineItems
    payment_flows: payment_flows.PaymentFlows
    payment_orders: payment_orders.PaymentOrders
    payment_references: payment_references.PaymentReferences
    returns: returns.Returns
    transactions: transactions.Transactions
    validations: validations.Validations
    paper_items: paper_items.PaperItems
    webhooks: webhooks.Webhooks
    virtual_accounts: virtual_accounts.VirtualAccounts
    bulk_requests: bulk_requests.BulkRequests
    bulk_results: bulk_results.BulkResults
    ledger_account_settlements: ledger_account_settlements.LedgerAccountSettlements
    foreign_exchange_quotes: foreign_exchange_quotes.ForeignExchangeQuotes
    connection_legal_entities: connection_legal_entities.ConnectionLegalEntities
    legal_entities: legal_entities.LegalEntities
    legal_entity_associations: legal_entity_associations.LegalEntityAssociations
    with_raw_response: ModernTreasuryWithRawResponse
    with_streaming_response: ModernTreasuryWithStreamedResponse

    # client options
    api_key: str
    organization_id: str
    webhook_key: str | None

    def __init__(
        self,
        *,
        api_key: str | None = None,
        organization_id: str | None = None,
        webhook_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous Modern Treasury client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `api_key` from `MODERN_TREASURY_API_KEY`
        - `organization_id` from `MODERN_TREASURY_ORGANIZATION_ID`
        - `webhook_key` from `MODERN_TREASURY_WEBHOOK_KEY`
        """
        if api_key is None:
            api_key = os.environ.get("MODERN_TREASURY_API_KEY")
        if api_key is None:
            raise ModernTreasuryError(
                "The api_key client option must be set either by passing api_key to the client or by setting the MODERN_TREASURY_API_KEY environment variable"
            )
        self.api_key = api_key

        if organization_id is None:
            organization_id = os.environ.get("MODERN_TREASURY_ORGANIZATION_ID")
        if organization_id is None:
            raise ModernTreasuryError(
                "The organization_id client option must be set either by passing organization_id to the client or by setting the MODERN_TREASURY_ORGANIZATION_ID environment variable"
            )
        self.organization_id = organization_id

        if webhook_key is None:
            webhook_key = os.environ.get("MODERN_TREASURY_WEBHOOK_KEY")
        self.webhook_key = webhook_key

        if base_url is None:
            base_url = os.environ.get("MODERN_TREASURY_BASE_URL")
        if base_url is None:
            base_url = f"https://app.moderntreasury.com"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self._idempotency_header = "Idempotency-Key"

        self.connections = connections.Connections(self)
        self.counterparties = counterparties.Counterparties(self)
        self.events = events.Events(self)
        self.expected_payments = expected_payments.ExpectedPayments(self)
        self.external_accounts = external_accounts.ExternalAccounts(self)
        self.incoming_payment_details = incoming_payment_details.IncomingPaymentDetails(self)
        self.invoices = invoices.Invoices(self)
        self.documents = documents.Documents(self)
        self.account_collection_flows = account_collection_flows.AccountCollectionFlows(self)
        self.account_details = account_details.AccountDetails(self)
        self.routing_details = routing_details.RoutingDetails(self)
        self.internal_accounts = internal_accounts.InternalAccounts(self)
        self.ledgers = ledgers.Ledgers(self)
        self.ledgerable_events = ledgerable_events.LedgerableEvents(self)
        self.ledger_account_categories = ledger_account_categories.LedgerAccountCategories(self)
        self.ledger_accounts = ledger_accounts.LedgerAccounts(self)
        self.ledger_account_balance_monitors = ledger_account_balance_monitors.LedgerAccountBalanceMonitors(self)
        self.ledger_account_statements = ledger_account_statements.LedgerAccountStatements(self)
        self.ledger_entries = ledger_entries.LedgerEntries(self)
        self.ledger_event_handlers = ledger_event_handlers.LedgerEventHandlers(self)
        self.ledger_transactions = ledger_transactions.LedgerTransactions(self)
        self.line_items = line_items.LineItems(self)
        self.payment_flows = payment_flows.PaymentFlows(self)
        self.payment_orders = payment_orders.PaymentOrders(self)
        self.payment_references = payment_references.PaymentReferences(self)
        self.returns = returns.Returns(self)
        self.transactions = transactions.Transactions(self)
        self.validations = validations.Validations(self)
        self.paper_items = paper_items.PaperItems(self)
        self.webhooks = webhooks.Webhooks(self)
        self.virtual_accounts = virtual_accounts.VirtualAccounts(self)
        self.bulk_requests = bulk_requests.BulkRequests(self)
        self.bulk_results = bulk_results.BulkResults(self)
        self.ledger_account_settlements = ledger_account_settlements.LedgerAccountSettlements(self)
        self.foreign_exchange_quotes = foreign_exchange_quotes.ForeignExchangeQuotes(self)
        self.connection_legal_entities = connection_legal_entities.ConnectionLegalEntities(self)
        self.legal_entities = legal_entities.LegalEntities(self)
        self.legal_entity_associations = legal_entity_associations.LegalEntityAssociations(self)
        self.with_raw_response = ModernTreasuryWithRawResponse(self)
        self.with_streaming_response = ModernTreasuryWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="brackets")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        credentials = f"{self.organization_id}:{self.api_key}".encode("ascii")
        header = f"Basic {base64.b64encode(credentials).decode('ascii')}"
        return {"Authorization": header}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        organization_id: str | None = None,
        webhook_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = NOT_GIVEN,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = NOT_GIVEN,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            organization_id=organization_id or self.organization_id,
            webhook_key=webhook_key or self.webhook_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    def ping(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PingResponse:
        """
        A test endpoint often used to confirm credentials and headers are being passed
        in correctly.
        """
        return self.get(
            "/api/ping",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PingResponse,
        )

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncModernTreasury(AsyncAPIClient):
    connections: connections.AsyncConnections
    counterparties: counterparties.AsyncCounterparties
    events: events.AsyncEvents
    expected_payments: expected_payments.AsyncExpectedPayments
    external_accounts: external_accounts.AsyncExternalAccounts
    incoming_payment_details: incoming_payment_details.AsyncIncomingPaymentDetails
    invoices: invoices.AsyncInvoices
    documents: documents.AsyncDocuments
    account_collection_flows: account_collection_flows.AsyncAccountCollectionFlows
    account_details: account_details.AsyncAccountDetails
    routing_details: routing_details.AsyncRoutingDetails
    internal_accounts: internal_accounts.AsyncInternalAccounts
    ledgers: ledgers.AsyncLedgers
    ledgerable_events: ledgerable_events.AsyncLedgerableEvents
    ledger_account_categories: ledger_account_categories.AsyncLedgerAccountCategories
    ledger_accounts: ledger_accounts.AsyncLedgerAccounts
    ledger_account_balance_monitors: ledger_account_balance_monitors.AsyncLedgerAccountBalanceMonitors
    ledger_account_statements: ledger_account_statements.AsyncLedgerAccountStatements
    ledger_entries: ledger_entries.AsyncLedgerEntries
    ledger_event_handlers: ledger_event_handlers.AsyncLedgerEventHandlers
    ledger_transactions: ledger_transactions.AsyncLedgerTransactions
    line_items: line_items.AsyncLineItems
    payment_flows: payment_flows.AsyncPaymentFlows
    payment_orders: payment_orders.AsyncPaymentOrders
    payment_references: payment_references.AsyncPaymentReferences
    returns: returns.AsyncReturns
    transactions: transactions.AsyncTransactions
    validations: validations.AsyncValidations
    paper_items: paper_items.AsyncPaperItems
    webhooks: webhooks.AsyncWebhooks
    virtual_accounts: virtual_accounts.AsyncVirtualAccounts
    bulk_requests: bulk_requests.AsyncBulkRequests
    bulk_results: bulk_results.AsyncBulkResults
    ledger_account_settlements: ledger_account_settlements.AsyncLedgerAccountSettlements
    foreign_exchange_quotes: foreign_exchange_quotes.AsyncForeignExchangeQuotes
    connection_legal_entities: connection_legal_entities.AsyncConnectionLegalEntities
    legal_entities: legal_entities.AsyncLegalEntities
    legal_entity_associations: legal_entity_associations.AsyncLegalEntityAssociations
    with_raw_response: AsyncModernTreasuryWithRawResponse
    with_streaming_response: AsyncModernTreasuryWithStreamedResponse

    # client options
    api_key: str
    organization_id: str
    webhook_key: str | None

    def __init__(
        self,
        *,
        api_key: str | None = None,
        organization_id: str | None = None,
        webhook_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async Modern Treasury client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `api_key` from `MODERN_TREASURY_API_KEY`
        - `organization_id` from `MODERN_TREASURY_ORGANIZATION_ID`
        - `webhook_key` from `MODERN_TREASURY_WEBHOOK_KEY`
        """
        if api_key is None:
            api_key = os.environ.get("MODERN_TREASURY_API_KEY")
        if api_key is None:
            raise ModernTreasuryError(
                "The api_key client option must be set either by passing api_key to the client or by setting the MODERN_TREASURY_API_KEY environment variable"
            )
        self.api_key = api_key

        if organization_id is None:
            organization_id = os.environ.get("MODERN_TREASURY_ORGANIZATION_ID")
        if organization_id is None:
            raise ModernTreasuryError(
                "The organization_id client option must be set either by passing organization_id to the client or by setting the MODERN_TREASURY_ORGANIZATION_ID environment variable"
            )
        self.organization_id = organization_id

        if webhook_key is None:
            webhook_key = os.environ.get("MODERN_TREASURY_WEBHOOK_KEY")
        self.webhook_key = webhook_key

        if base_url is None:
            base_url = os.environ.get("MODERN_TREASURY_BASE_URL")
        if base_url is None:
            base_url = f"https://app.moderntreasury.com"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self._idempotency_header = "Idempotency-Key"

        self.connections = connections.AsyncConnections(self)
        self.counterparties = counterparties.AsyncCounterparties(self)
        self.events = events.AsyncEvents(self)
        self.expected_payments = expected_payments.AsyncExpectedPayments(self)
        self.external_accounts = external_accounts.AsyncExternalAccounts(self)
        self.incoming_payment_details = incoming_payment_details.AsyncIncomingPaymentDetails(self)
        self.invoices = invoices.AsyncInvoices(self)
        self.documents = documents.AsyncDocuments(self)
        self.account_collection_flows = account_collection_flows.AsyncAccountCollectionFlows(self)
        self.account_details = account_details.AsyncAccountDetails(self)
        self.routing_details = routing_details.AsyncRoutingDetails(self)
        self.internal_accounts = internal_accounts.AsyncInternalAccounts(self)
        self.ledgers = ledgers.AsyncLedgers(self)
        self.ledgerable_events = ledgerable_events.AsyncLedgerableEvents(self)
        self.ledger_account_categories = ledger_account_categories.AsyncLedgerAccountCategories(self)
        self.ledger_accounts = ledger_accounts.AsyncLedgerAccounts(self)
        self.ledger_account_balance_monitors = ledger_account_balance_monitors.AsyncLedgerAccountBalanceMonitors(self)
        self.ledger_account_statements = ledger_account_statements.AsyncLedgerAccountStatements(self)
        self.ledger_entries = ledger_entries.AsyncLedgerEntries(self)
        self.ledger_event_handlers = ledger_event_handlers.AsyncLedgerEventHandlers(self)
        self.ledger_transactions = ledger_transactions.AsyncLedgerTransactions(self)
        self.line_items = line_items.AsyncLineItems(self)
        self.payment_flows = payment_flows.AsyncPaymentFlows(self)
        self.payment_orders = payment_orders.AsyncPaymentOrders(self)
        self.payment_references = payment_references.AsyncPaymentReferences(self)
        self.returns = returns.AsyncReturns(self)
        self.transactions = transactions.AsyncTransactions(self)
        self.validations = validations.AsyncValidations(self)
        self.paper_items = paper_items.AsyncPaperItems(self)
        self.webhooks = webhooks.AsyncWebhooks(self)
        self.virtual_accounts = virtual_accounts.AsyncVirtualAccounts(self)
        self.bulk_requests = bulk_requests.AsyncBulkRequests(self)
        self.bulk_results = bulk_results.AsyncBulkResults(self)
        self.ledger_account_settlements = ledger_account_settlements.AsyncLedgerAccountSettlements(self)
        self.foreign_exchange_quotes = foreign_exchange_quotes.AsyncForeignExchangeQuotes(self)
        self.connection_legal_entities = connection_legal_entities.AsyncConnectionLegalEntities(self)
        self.legal_entities = legal_entities.AsyncLegalEntities(self)
        self.legal_entity_associations = legal_entity_associations.AsyncLegalEntityAssociations(self)
        self.with_raw_response = AsyncModernTreasuryWithRawResponse(self)
        self.with_streaming_response = AsyncModernTreasuryWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="brackets")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        credentials = f"{self.organization_id}:{self.api_key}".encode("ascii")
        header = f"Basic {base64.b64encode(credentials).decode('ascii')}"
        return {"Authorization": header}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        organization_id: str | None = None,
        webhook_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = NOT_GIVEN,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = NOT_GIVEN,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            organization_id=organization_id or self.organization_id,
            webhook_key=webhook_key or self.webhook_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    async def ping(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PingResponse:
        """
        A test endpoint often used to confirm credentials and headers are being passed
        in correctly.
        """
        return await self.get(
            "/api/ping",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PingResponse,
        )

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class ModernTreasuryWithRawResponse:
    def __init__(self, client: ModernTreasury) -> None:
        self.connections = connections.ConnectionsWithRawResponse(client.connections)
        self.counterparties = counterparties.CounterpartiesWithRawResponse(client.counterparties)
        self.events = events.EventsWithRawResponse(client.events)
        self.expected_payments = expected_payments.ExpectedPaymentsWithRawResponse(client.expected_payments)
        self.external_accounts = external_accounts.ExternalAccountsWithRawResponse(client.external_accounts)
        self.incoming_payment_details = incoming_payment_details.IncomingPaymentDetailsWithRawResponse(
            client.incoming_payment_details
        )
        self.invoices = invoices.InvoicesWithRawResponse(client.invoices)
        self.documents = documents.DocumentsWithRawResponse(client.documents)
        self.account_collection_flows = account_collection_flows.AccountCollectionFlowsWithRawResponse(
            client.account_collection_flows
        )
        self.account_details = account_details.AccountDetailsWithRawResponse(client.account_details)
        self.routing_details = routing_details.RoutingDetailsWithRawResponse(client.routing_details)
        self.internal_accounts = internal_accounts.InternalAccountsWithRawResponse(client.internal_accounts)
        self.ledgers = ledgers.LedgersWithRawResponse(client.ledgers)
        self.ledgerable_events = ledgerable_events.LedgerableEventsWithRawResponse(client.ledgerable_events)
        self.ledger_account_categories = ledger_account_categories.LedgerAccountCategoriesWithRawResponse(
            client.ledger_account_categories
        )
        self.ledger_accounts = ledger_accounts.LedgerAccountsWithRawResponse(client.ledger_accounts)
        self.ledger_account_balance_monitors = (
            ledger_account_balance_monitors.LedgerAccountBalanceMonitorsWithRawResponse(
                client.ledger_account_balance_monitors
            )
        )
        self.ledger_account_statements = ledger_account_statements.LedgerAccountStatementsWithRawResponse(
            client.ledger_account_statements
        )
        self.ledger_entries = ledger_entries.LedgerEntriesWithRawResponse(client.ledger_entries)
        self.ledger_event_handlers = ledger_event_handlers.LedgerEventHandlersWithRawResponse(
            client.ledger_event_handlers
        )
        self.ledger_transactions = ledger_transactions.LedgerTransactionsWithRawResponse(client.ledger_transactions)
        self.line_items = line_items.LineItemsWithRawResponse(client.line_items)
        self.payment_flows = payment_flows.PaymentFlowsWithRawResponse(client.payment_flows)
        self.payment_orders = payment_orders.PaymentOrdersWithRawResponse(client.payment_orders)
        self.payment_references = payment_references.PaymentReferencesWithRawResponse(client.payment_references)
        self.returns = returns.ReturnsWithRawResponse(client.returns)
        self.transactions = transactions.TransactionsWithRawResponse(client.transactions)
        self.validations = validations.ValidationsWithRawResponse(client.validations)
        self.paper_items = paper_items.PaperItemsWithRawResponse(client.paper_items)
        self.virtual_accounts = virtual_accounts.VirtualAccountsWithRawResponse(client.virtual_accounts)
        self.bulk_requests = bulk_requests.BulkRequestsWithRawResponse(client.bulk_requests)
        self.bulk_results = bulk_results.BulkResultsWithRawResponse(client.bulk_results)
        self.ledger_account_settlements = ledger_account_settlements.LedgerAccountSettlementsWithRawResponse(
            client.ledger_account_settlements
        )
        self.foreign_exchange_quotes = foreign_exchange_quotes.ForeignExchangeQuotesWithRawResponse(
            client.foreign_exchange_quotes
        )
        self.connection_legal_entities = connection_legal_entities.ConnectionLegalEntitiesWithRawResponse(
            client.connection_legal_entities
        )
        self.legal_entities = legal_entities.LegalEntitiesWithRawResponse(client.legal_entities)
        self.legal_entity_associations = legal_entity_associations.LegalEntityAssociationsWithRawResponse(
            client.legal_entity_associations
        )

        self.ping = _legacy_response.to_raw_response_wrapper(
            client.ping,
        )


class AsyncModernTreasuryWithRawResponse:
    def __init__(self, client: AsyncModernTreasury) -> None:
        self.connections = connections.AsyncConnectionsWithRawResponse(client.connections)
        self.counterparties = counterparties.AsyncCounterpartiesWithRawResponse(client.counterparties)
        self.events = events.AsyncEventsWithRawResponse(client.events)
        self.expected_payments = expected_payments.AsyncExpectedPaymentsWithRawResponse(client.expected_payments)
        self.external_accounts = external_accounts.AsyncExternalAccountsWithRawResponse(client.external_accounts)
        self.incoming_payment_details = incoming_payment_details.AsyncIncomingPaymentDetailsWithRawResponse(
            client.incoming_payment_details
        )
        self.invoices = invoices.AsyncInvoicesWithRawResponse(client.invoices)
        self.documents = documents.AsyncDocumentsWithRawResponse(client.documents)
        self.account_collection_flows = account_collection_flows.AsyncAccountCollectionFlowsWithRawResponse(
            client.account_collection_flows
        )
        self.account_details = account_details.AsyncAccountDetailsWithRawResponse(client.account_details)
        self.routing_details = routing_details.AsyncRoutingDetailsWithRawResponse(client.routing_details)
        self.internal_accounts = internal_accounts.AsyncInternalAccountsWithRawResponse(client.internal_accounts)
        self.ledgers = ledgers.AsyncLedgersWithRawResponse(client.ledgers)
        self.ledgerable_events = ledgerable_events.AsyncLedgerableEventsWithRawResponse(client.ledgerable_events)
        self.ledger_account_categories = ledger_account_categories.AsyncLedgerAccountCategoriesWithRawResponse(
            client.ledger_account_categories
        )
        self.ledger_accounts = ledger_accounts.AsyncLedgerAccountsWithRawResponse(client.ledger_accounts)
        self.ledger_account_balance_monitors = (
            ledger_account_balance_monitors.AsyncLedgerAccountBalanceMonitorsWithRawResponse(
                client.ledger_account_balance_monitors
            )
        )
        self.ledger_account_statements = ledger_account_statements.AsyncLedgerAccountStatementsWithRawResponse(
            client.ledger_account_statements
        )
        self.ledger_entries = ledger_entries.AsyncLedgerEntriesWithRawResponse(client.ledger_entries)
        self.ledger_event_handlers = ledger_event_handlers.AsyncLedgerEventHandlersWithRawResponse(
            client.ledger_event_handlers
        )
        self.ledger_transactions = ledger_transactions.AsyncLedgerTransactionsWithRawResponse(
            client.ledger_transactions
        )
        self.line_items = line_items.AsyncLineItemsWithRawResponse(client.line_items)
        self.payment_flows = payment_flows.AsyncPaymentFlowsWithRawResponse(client.payment_flows)
        self.payment_orders = payment_orders.AsyncPaymentOrdersWithRawResponse(client.payment_orders)
        self.payment_references = payment_references.AsyncPaymentReferencesWithRawResponse(client.payment_references)
        self.returns = returns.AsyncReturnsWithRawResponse(client.returns)
        self.transactions = transactions.AsyncTransactionsWithRawResponse(client.transactions)
        self.validations = validations.AsyncValidationsWithRawResponse(client.validations)
        self.paper_items = paper_items.AsyncPaperItemsWithRawResponse(client.paper_items)
        self.virtual_accounts = virtual_accounts.AsyncVirtualAccountsWithRawResponse(client.virtual_accounts)
        self.bulk_requests = bulk_requests.AsyncBulkRequestsWithRawResponse(client.bulk_requests)
        self.bulk_results = bulk_results.AsyncBulkResultsWithRawResponse(client.bulk_results)
        self.ledger_account_settlements = ledger_account_settlements.AsyncLedgerAccountSettlementsWithRawResponse(
            client.ledger_account_settlements
        )
        self.foreign_exchange_quotes = foreign_exchange_quotes.AsyncForeignExchangeQuotesWithRawResponse(
            client.foreign_exchange_quotes
        )
        self.connection_legal_entities = connection_legal_entities.AsyncConnectionLegalEntitiesWithRawResponse(
            client.connection_legal_entities
        )
        self.legal_entities = legal_entities.AsyncLegalEntitiesWithRawResponse(client.legal_entities)
        self.legal_entity_associations = legal_entity_associations.AsyncLegalEntityAssociationsWithRawResponse(
            client.legal_entity_associations
        )

        self.ping = _legacy_response.async_to_raw_response_wrapper(
            client.ping,
        )


class ModernTreasuryWithStreamedResponse:
    def __init__(self, client: ModernTreasury) -> None:
        self.connections = connections.ConnectionsWithStreamingResponse(client.connections)
        self.counterparties = counterparties.CounterpartiesWithStreamingResponse(client.counterparties)
        self.events = events.EventsWithStreamingResponse(client.events)
        self.expected_payments = expected_payments.ExpectedPaymentsWithStreamingResponse(client.expected_payments)
        self.external_accounts = external_accounts.ExternalAccountsWithStreamingResponse(client.external_accounts)
        self.incoming_payment_details = incoming_payment_details.IncomingPaymentDetailsWithStreamingResponse(
            client.incoming_payment_details
        )
        self.invoices = invoices.InvoicesWithStreamingResponse(client.invoices)
        self.documents = documents.DocumentsWithStreamingResponse(client.documents)
        self.account_collection_flows = account_collection_flows.AccountCollectionFlowsWithStreamingResponse(
            client.account_collection_flows
        )
        self.account_details = account_details.AccountDetailsWithStreamingResponse(client.account_details)
        self.routing_details = routing_details.RoutingDetailsWithStreamingResponse(client.routing_details)
        self.internal_accounts = internal_accounts.InternalAccountsWithStreamingResponse(client.internal_accounts)
        self.ledgers = ledgers.LedgersWithStreamingResponse(client.ledgers)
        self.ledgerable_events = ledgerable_events.LedgerableEventsWithStreamingResponse(client.ledgerable_events)
        self.ledger_account_categories = ledger_account_categories.LedgerAccountCategoriesWithStreamingResponse(
            client.ledger_account_categories
        )
        self.ledger_accounts = ledger_accounts.LedgerAccountsWithStreamingResponse(client.ledger_accounts)
        self.ledger_account_balance_monitors = (
            ledger_account_balance_monitors.LedgerAccountBalanceMonitorsWithStreamingResponse(
                client.ledger_account_balance_monitors
            )
        )
        self.ledger_account_statements = ledger_account_statements.LedgerAccountStatementsWithStreamingResponse(
            client.ledger_account_statements
        )
        self.ledger_entries = ledger_entries.LedgerEntriesWithStreamingResponse(client.ledger_entries)
        self.ledger_event_handlers = ledger_event_handlers.LedgerEventHandlersWithStreamingResponse(
            client.ledger_event_handlers
        )
        self.ledger_transactions = ledger_transactions.LedgerTransactionsWithStreamingResponse(
            client.ledger_transactions
        )
        self.line_items = line_items.LineItemsWithStreamingResponse(client.line_items)
        self.payment_flows = payment_flows.PaymentFlowsWithStreamingResponse(client.payment_flows)
        self.payment_orders = payment_orders.PaymentOrdersWithStreamingResponse(client.payment_orders)
        self.payment_references = payment_references.PaymentReferencesWithStreamingResponse(client.payment_references)
        self.returns = returns.ReturnsWithStreamingResponse(client.returns)
        self.transactions = transactions.TransactionsWithStreamingResponse(client.transactions)
        self.validations = validations.ValidationsWithStreamingResponse(client.validations)
        self.paper_items = paper_items.PaperItemsWithStreamingResponse(client.paper_items)
        self.virtual_accounts = virtual_accounts.VirtualAccountsWithStreamingResponse(client.virtual_accounts)
        self.bulk_requests = bulk_requests.BulkRequestsWithStreamingResponse(client.bulk_requests)
        self.bulk_results = bulk_results.BulkResultsWithStreamingResponse(client.bulk_results)
        self.ledger_account_settlements = ledger_account_settlements.LedgerAccountSettlementsWithStreamingResponse(
            client.ledger_account_settlements
        )
        self.foreign_exchange_quotes = foreign_exchange_quotes.ForeignExchangeQuotesWithStreamingResponse(
            client.foreign_exchange_quotes
        )
        self.connection_legal_entities = connection_legal_entities.ConnectionLegalEntitiesWithStreamingResponse(
            client.connection_legal_entities
        )
        self.legal_entities = legal_entities.LegalEntitiesWithStreamingResponse(client.legal_entities)
        self.legal_entity_associations = legal_entity_associations.LegalEntityAssociationsWithStreamingResponse(
            client.legal_entity_associations
        )

        self.ping = to_streamed_response_wrapper(
            client.ping,
        )


class AsyncModernTreasuryWithStreamedResponse:
    def __init__(self, client: AsyncModernTreasury) -> None:
        self.connections = connections.AsyncConnectionsWithStreamingResponse(client.connections)
        self.counterparties = counterparties.AsyncCounterpartiesWithStreamingResponse(client.counterparties)
        self.events = events.AsyncEventsWithStreamingResponse(client.events)
        self.expected_payments = expected_payments.AsyncExpectedPaymentsWithStreamingResponse(client.expected_payments)
        self.external_accounts = external_accounts.AsyncExternalAccountsWithStreamingResponse(client.external_accounts)
        self.incoming_payment_details = incoming_payment_details.AsyncIncomingPaymentDetailsWithStreamingResponse(
            client.incoming_payment_details
        )
        self.invoices = invoices.AsyncInvoicesWithStreamingResponse(client.invoices)
        self.documents = documents.AsyncDocumentsWithStreamingResponse(client.documents)
        self.account_collection_flows = account_collection_flows.AsyncAccountCollectionFlowsWithStreamingResponse(
            client.account_collection_flows
        )
        self.account_details = account_details.AsyncAccountDetailsWithStreamingResponse(client.account_details)
        self.routing_details = routing_details.AsyncRoutingDetailsWithStreamingResponse(client.routing_details)
        self.internal_accounts = internal_accounts.AsyncInternalAccountsWithStreamingResponse(client.internal_accounts)
        self.ledgers = ledgers.AsyncLedgersWithStreamingResponse(client.ledgers)
        self.ledgerable_events = ledgerable_events.AsyncLedgerableEventsWithStreamingResponse(client.ledgerable_events)
        self.ledger_account_categories = ledger_account_categories.AsyncLedgerAccountCategoriesWithStreamingResponse(
            client.ledger_account_categories
        )
        self.ledger_accounts = ledger_accounts.AsyncLedgerAccountsWithStreamingResponse(client.ledger_accounts)
        self.ledger_account_balance_monitors = (
            ledger_account_balance_monitors.AsyncLedgerAccountBalanceMonitorsWithStreamingResponse(
                client.ledger_account_balance_monitors
            )
        )
        self.ledger_account_statements = ledger_account_statements.AsyncLedgerAccountStatementsWithStreamingResponse(
            client.ledger_account_statements
        )
        self.ledger_entries = ledger_entries.AsyncLedgerEntriesWithStreamingResponse(client.ledger_entries)
        self.ledger_event_handlers = ledger_event_handlers.AsyncLedgerEventHandlersWithStreamingResponse(
            client.ledger_event_handlers
        )
        self.ledger_transactions = ledger_transactions.AsyncLedgerTransactionsWithStreamingResponse(
            client.ledger_transactions
        )
        self.line_items = line_items.AsyncLineItemsWithStreamingResponse(client.line_items)
        self.payment_flows = payment_flows.AsyncPaymentFlowsWithStreamingResponse(client.payment_flows)
        self.payment_orders = payment_orders.AsyncPaymentOrdersWithStreamingResponse(client.payment_orders)
        self.payment_references = payment_references.AsyncPaymentReferencesWithStreamingResponse(
            client.payment_references
        )
        self.returns = returns.AsyncReturnsWithStreamingResponse(client.returns)
        self.transactions = transactions.AsyncTransactionsWithStreamingResponse(client.transactions)
        self.validations = validations.AsyncValidationsWithStreamingResponse(client.validations)
        self.paper_items = paper_items.AsyncPaperItemsWithStreamingResponse(client.paper_items)
        self.virtual_accounts = virtual_accounts.AsyncVirtualAccountsWithStreamingResponse(client.virtual_accounts)
        self.bulk_requests = bulk_requests.AsyncBulkRequestsWithStreamingResponse(client.bulk_requests)
        self.bulk_results = bulk_results.AsyncBulkResultsWithStreamingResponse(client.bulk_results)
        self.ledger_account_settlements = ledger_account_settlements.AsyncLedgerAccountSettlementsWithStreamingResponse(
            client.ledger_account_settlements
        )
        self.foreign_exchange_quotes = foreign_exchange_quotes.AsyncForeignExchangeQuotesWithStreamingResponse(
            client.foreign_exchange_quotes
        )
        self.connection_legal_entities = connection_legal_entities.AsyncConnectionLegalEntitiesWithStreamingResponse(
            client.connection_legal_entities
        )
        self.legal_entities = legal_entities.AsyncLegalEntitiesWithStreamingResponse(client.legal_entities)
        self.legal_entity_associations = legal_entity_associations.AsyncLegalEntityAssociationsWithStreamingResponse(
            client.legal_entity_associations
        )

        self.ping = async_to_streamed_response_wrapper(
            client.ping,
        )


Client = ModernTreasury

AsyncClient = AsyncModernTreasury



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import Required, TypedDict

__all__ = ["LedgerableEventCreateParams"]


class LedgerableEventCreateParams(TypedDict, total=False):
    name: Required[str]
    """Name of the ledgerable event."""

    custom_data: Optional[object]
    """Additionally data to be used by the Ledger Event Handler."""

    description: Optional[str]
    """Description of the ledgerable event."""

    metadata: Dict[str, str]
    """Additional data represented as key-value pairs.

    Both the key and value must be strings.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from modern_treasury import ModernTreasury, AsyncModernTreasury
from modern_treasury.types import AccountDetail
from modern_treasury.pagination import SyncPage, AsyncPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestAccountDetails:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: ModernTreasury) -> None:
        account_detail = client.account_details.create(
            account_id="account_id",
            accounts_type="external_accounts",
            account_number="account_number",
        )
        assert_matches_type(AccountDetail, account_detail, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: ModernTreasury) -> None:
        account_detail = client.account_details.create(
            account_id="account_id",
            accounts_type="external_accounts",
            account_number="account_number",
            account_number_type="au_number",
        )
        assert_matches_type(AccountDetail, account_detail, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: ModernTreasury) -> None:
        response = client.account_details.with_raw_response.create(
            account_id="account_id",
            accounts_type="external_accounts",
            account_number="account_number",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account_detail = response.parse()
        assert_matches_type(AccountDetail, account_detail, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: ModernTreasury) -> None:
        with client.account_details.with_streaming_response.create(
            account_id="account_id",
            accounts_type="external_accounts",
            account_number="account_number",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account_detail = response.parse()
            assert_matches_type(AccountDetail, account_detail, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_create(self, client: ModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            client.account_details.with_raw_response.create(
                account_id="",
                accounts_type="external_accounts",
                account_number="account_number",
            )

    @parametrize
    def test_method_retrieve(self, client: ModernTreasury) -> None:
        account_detail = client.account_details.retrieve(
            id="id",
            accounts_type="external_accounts",
            account_id="account_id",
        )
        assert_matches_type(AccountDetail, account_detail, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: ModernTreasury) -> None:
        response = client.account_details.with_raw_response.retrieve(
            id="id",
            accounts_type="external_accounts",
            account_id="account_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account_detail = response.parse()
        assert_matches_type(AccountDetail, account_detail, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: ModernTreasury) -> None:
        with client.account_details.with_streaming_response.retrieve(
            id="id",
            accounts_type="external_accounts",
            account_id="account_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account_detail = response.parse()
            assert_matches_type(AccountDetail, account_detail, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: ModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            client.account_details.with_raw_response.retrieve(
                id="id",
                accounts_type="external_accounts",
                account_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.account_details.with_raw_response.retrieve(
                id="",
                accounts_type="external_accounts",
                account_id="account_id",
            )

    @parametrize
    def test_method_list(self, client: ModernTreasury) -> None:
        account_detail = client.account_details.list(
            account_id="account_id",
            accounts_type="external_accounts",
        )
        assert_matches_type(SyncPage[AccountDetail], account_detail, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: ModernTreasury) -> None:
        account_detail = client.account_details.list(
            account_id="account_id",
            accounts_type="external_accounts",
            after_cursor="after_cursor",
            per_page=0,
        )
        assert_matches_type(SyncPage[AccountDetail], account_detail, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: ModernTreasury) -> None:
        response = client.account_details.with_raw_response.list(
            account_id="account_id",
            accounts_type="external_accounts",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account_detail = response.parse()
        assert_matches_type(SyncPage[AccountDetail], account_detail, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: ModernTreasury) -> None:
        with client.account_details.with_streaming_response.list(
            account_id="account_id",
            accounts_type="external_accounts",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account_detail = response.parse()
            assert_matches_type(SyncPage[AccountDetail], account_detail, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list(self, client: ModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            client.account_details.with_raw_response.list(
                account_id="",
                accounts_type="external_accounts",
            )

    @parametrize
    def test_method_delete(self, client: ModernTreasury) -> None:
        account_detail = client.account_details.delete(
            id="id",
            accounts_type="external_accounts",
            account_id="account_id",
        )
        assert account_detail is None

    @parametrize
    def test_raw_response_delete(self, client: ModernTreasury) -> None:
        response = client.account_details.with_raw_response.delete(
            id="id",
            accounts_type="external_accounts",
            account_id="account_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account_detail = response.parse()
        assert account_detail is None

    @parametrize
    def test_streaming_response_delete(self, client: ModernTreasury) -> None:
        with client.account_details.with_streaming_response.delete(
            id="id",
            accounts_type="external_accounts",
            account_id="account_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account_detail = response.parse()
            assert account_detail is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: ModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            client.account_details.with_raw_response.delete(
                id="id",
                accounts_type="external_accounts",
                account_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.account_details.with_raw_response.delete(
                id="",
                accounts_type="external_accounts",
                account_id="account_id",
            )


class TestAsyncAccountDetails:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncModernTreasury) -> None:
        account_detail = await async_client.account_details.create(
            account_id="account_id",
            accounts_type="external_accounts",
            account_number="account_number",
        )
        assert_matches_type(AccountDetail, account_detail, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncModernTreasury) -> None:
        account_detail = await async_client.account_details.create(
            account_id="account_id",
            accounts_type="external_accounts",
            account_number="account_number",
            account_number_type="au_number",
        )
        assert_matches_type(AccountDetail, account_detail, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.account_details.with_raw_response.create(
            account_id="account_id",
            accounts_type="external_accounts",
            account_number="account_number",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account_detail = response.parse()
        assert_matches_type(AccountDetail, account_detail, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.account_details.with_streaming_response.create(
            account_id="account_id",
            accounts_type="external_accounts",
            account_number="account_number",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account_detail = await response.parse()
            assert_matches_type(AccountDetail, account_detail, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_create(self, async_client: AsyncModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            await async_client.account_details.with_raw_response.create(
                account_id="",
                accounts_type="external_accounts",
                account_number="account_number",
            )

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncModernTreasury) -> None:
        account_detail = await async_client.account_details.retrieve(
            id="id",
            accounts_type="external_accounts",
            account_id="account_id",
        )
        assert_matches_type(AccountDetail, account_detail, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.account_details.with_raw_response.retrieve(
            id="id",
            accounts_type="external_accounts",
            account_id="account_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account_detail = response.parse()
        assert_matches_type(AccountDetail, account_detail, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.account_details.with_streaming_response.retrieve(
            id="id",
            accounts_type="external_accounts",
            account_id="account_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account_detail = await response.parse()
            assert_matches_type(AccountDetail, account_detail, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            await async_client.account_details.with_raw_response.retrieve(
                id="id",
                accounts_type="external_accounts",
                account_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.account_details.with_raw_response.retrieve(
                id="",
                accounts_type="external_accounts",
                account_id="account_id",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncModernTreasury) -> None:
        account_detail = await async_client.account_details.list(
            account_id="account_id",
            accounts_type="external_accounts",
        )
        assert_matches_type(AsyncPage[AccountDetail], account_detail, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncModernTreasury) -> None:
        account_detail = await async_client.account_details.list(
            account_id="account_id",
            accounts_type="external_accounts",
            after_cursor="after_cursor",
            per_page=0,
        )
        assert_matches_type(AsyncPage[AccountDetail], account_detail, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.account_details.with_raw_response.list(
            account_id="account_id",
            accounts_type="external_accounts",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account_detail = response.parse()
        assert_matches_type(AsyncPage[AccountDetail], account_detail, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.account_details.with_streaming_response.list(
            account_id="account_id",
            accounts_type="external_accounts",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account_detail = await response.parse()
            assert_matches_type(AsyncPage[AccountDetail], account_detail, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list(self, async_client: AsyncModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            await async_client.account_details.with_raw_response.list(
                account_id="",
                accounts_type="external_accounts",
            )

    @parametrize
    async def test_method_delete(self, async_client: AsyncModernTreasury) -> None:
        account_detail = await async_client.account_details.delete(
            id="id",
            accounts_type="external_accounts",
            account_id="account_id",
        )
        assert account_detail is None

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.account_details.with_raw_response.delete(
            id="id",
            accounts_type="external_accounts",
            account_id="account_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account_detail = response.parse()
        assert account_detail is None

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.account_details.with_streaming_response.delete(
            id="id",
            accounts_type="external_accounts",
            account_id="account_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account_detail = await response.parse()
            assert account_detail is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            await async_client.account_details.with_raw_response.delete(
                id="id",
                accounts_type="external_accounts",
                account_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.account_details.with_raw_response.delete(
                id="",
                accounts_type="external_accounts",
                account_id="account_id",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Optional

import httpx

from .. import _legacy_response
from ..types import (
    ledger_account_category_list_params,
    ledger_account_category_create_params,
    ledger_account_category_update_params,
    ledger_account_category_retrieve_params,
)
from .._types import NOT_GIVEN, Body, Query, Headers, NoneType, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import to_streamed_response_wrapper, async_to_streamed_response_wrapper
from ..pagination import SyncPage, AsyncPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.ledger_account_category import LedgerAccountCategory
from ..types.shared.transaction_direction import TransactionDirection

__all__ = ["LedgerAccountCategories", "AsyncLedgerAccountCategories"]


class LedgerAccountCategories(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> LedgerAccountCategoriesWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#accessing-raw-response-data-eg-headers
        """
        return LedgerAccountCategoriesWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LedgerAccountCategoriesWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#with_streaming_response
        """
        return LedgerAccountCategoriesWithStreamingResponse(self)

    def create(
        self,
        *,
        currency: str,
        ledger_id: str,
        name: str,
        normal_balance: TransactionDirection,
        currency_exponent: Optional[int] | NotGiven = NOT_GIVEN,
        description: Optional[str] | NotGiven = NOT_GIVEN,
        ledger_account_category_ids: List[str] | NotGiven = NOT_GIVEN,
        metadata: Dict[str, str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> LedgerAccountCategory:
        """
        Create a ledger account category.

        Args:
          currency: The currency of the ledger account category.

          ledger_id: The id of the ledger that this account category belongs to.

          name: The name of the ledger account category.

          normal_balance: The normal balance of the ledger account category.

          currency_exponent: The currency exponent of the ledger account category.

          description: The description of the ledger account category.

          ledger_account_category_ids: The array of ledger account category ids that this ledger account category
              should be a child of.

          metadata: Additional data represented as key-value pairs. Both the key and value must be
              strings.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        return self._post(
            "/api/ledger_account_categories",
            body=maybe_transform(
                {
                    "currency": currency,
                    "ledger_id": ledger_id,
                    "name": name,
                    "normal_balance": normal_balance,
                    "currency_exponent": currency_exponent,
                    "description": description,
                    "ledger_account_category_ids": ledger_account_category_ids,
                    "metadata": metadata,
                },
                ledger_account_category_create_params.LedgerAccountCategoryCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=LedgerAccountCategory,
        )

    def retrieve(
        self,
        id: str,
        *,
        balances: ledger_account_category_retrieve_params.Balances | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> LedgerAccountCategory:
        """
        Get the details on a single ledger account category.

        Args:
          balances: For example, if you want the balances as of a particular time (ISO8601), the
              encoded query string would be `balances%5Beffective_at%5D=2000-12-31T12:00:00Z`.
              The balances as of a time are inclusive of entries with that exact time.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/api/ledger_account_categories/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"balances": balances}, ledger_account_category_retrieve_params.LedgerAccountCategoryRetrieveParams
                ),
            ),
            cast_to=LedgerAccountCategory,
        )

    def update(
        self,
        id: str,
        *,
        description: Optional[str] | NotGiven = NOT_GIVEN,
        metadata: Dict[str, str] | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> LedgerAccountCategory:
        """
        Update the details of a ledger account category.

        Args:
          description: The description of the ledger account category.

          metadata: Additional data represented as key-value pairs. Both the key and value must be
              strings.

          name: The name of the ledger account category.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/api/ledger_account_categories/{id}",
            body=maybe_transform(
                {
                    "description": description,
                    "metadata": metadata,
                    "name": name,
                },
                ledger_account_category_update_params.LedgerAccountCategoryUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=LedgerAccountCategory,
        )

    def list(
        self,
        *,
        id: List[str] | NotGiven = NOT_GIVEN,
        after_cursor: Optional[str] | NotGiven = NOT_GIVEN,
        balances: ledger_account_category_list_params.Balances | NotGiven = NOT_GIVEN,
        currency: str | NotGiven = NOT_GIVEN,
        ledger_account_id: str | NotGiven = NOT_GIVEN,
        ledger_id: str | NotGiven = NOT_GIVEN,
        metadata: Dict[str, str] | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        parent_ledger_account_category_id: str | NotGiven = NOT_GIVEN,
        per_page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncPage[LedgerAccountCategory]:
        """
        Get a list of ledger account categories.

        Args:
          id: If you have specific IDs to retrieve in bulk, you can pass them as query
              parameters delimited with `id[]=`, for example `?id[]=123&id[]=abc`.

          balances: For example, if you want the balances as of a particular time (ISO8601), the
              encoded query string would be `balances%5Beffective_at%5D=2000-12-31T12:00:00Z`.
              The balances as of a time are inclusive of entries with that exact time.

          ledger_account_id: Query categories which contain a ledger account directly or through child
              categories.

          metadata: For example, if you want to query for records with metadata key `Type` and value
              `Loan`, the query would be `metadata%5BType%5D=Loan`. This encodes the query
              parameters.

          parent_ledger_account_category_id: Query categories that are nested underneath a parent category

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/ledger_account_categories",
            page=SyncPage[LedgerAccountCategory],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "after_cursor": after_cursor,
                        "balances": balances,
                        "currency": currency,
                        "ledger_account_id": ledger_account_id,
                        "ledger_id": ledger_id,
                        "metadata": metadata,
                        "name": name,
                        "parent_ledger_account_category_id": parent_ledger_account_category_id,
                        "per_page": per_page,
                    },
                    ledger_account_category_list_params.LedgerAccountCategoryListParams,
                ),
            ),
            model=LedgerAccountCategory,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> LedgerAccountCategory:
        """
        Delete a ledger account category.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/api/ledger_account_categories/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=LedgerAccountCategory,
        )

    def add_ledger_account(
        self,
        ledger_account_id: str,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> None:
        """
        Add a ledger account to a ledger account category.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not ledger_account_id:
            raise ValueError(f"Expected a non-empty value for `ledger_account_id` but received {ledger_account_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._put(
            f"/api/ledger_account_categories/{id}/ledger_accounts/{ledger_account_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=NoneType,
        )

    def add_nested_category(
        self,
        sub_category_id: str,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> None:
        """
        Add a ledger account category to a ledger account category.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not sub_category_id:
            raise ValueError(f"Expected a non-empty value for `sub_category_id` but received {sub_category_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._put(
            f"/api/ledger_account_categories/{id}/ledger_account_categories/{sub_category_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=NoneType,
        )

    def remove_ledger_account(
        self,
        ledger_account_id: str,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> None:
        """
        Remove a ledger account from a ledger account category.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not ledger_account_id:
            raise ValueError(f"Expected a non-empty value for `ledger_account_id` but received {ledger_account_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/api/ledger_account_categories/{id}/ledger_accounts/{ledger_account_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=NoneType,
        )

    def remove_nested_category(
        self,
        sub_category_id: str,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> None:
        """
        Delete a ledger account category from a ledger account category.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not sub_category_id:
            raise ValueError(f"Expected a non-empty value for `sub_category_id` but received {sub_category_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/api/ledger_account_categories/{id}/ledger_account_categories/{sub_category_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=NoneType,
        )


class AsyncLedgerAccountCategories(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncLedgerAccountCategoriesWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#accessing-raw-response-data-eg-headers
        """
        return AsyncLedgerAccountCategoriesWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLedgerAccountCategoriesWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#with_streaming_response
        """
        return AsyncLedgerAccountCategoriesWithStreamingResponse(self)

    async def create(
        self,
        *,
        currency: str,
        ledger_id: str,
        name: str,
        normal_balance: TransactionDirection,
        currency_exponent: Optional[int] | NotGiven = NOT_GIVEN,
        description: Optional[str] | NotGiven = NOT_GIVEN,
        ledger_account_category_ids: List[str] | NotGiven = NOT_GIVEN,
        metadata: Dict[str, str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> LedgerAccountCategory:
        """
        Create a ledger account category.

        Args:
          currency: The currency of the ledger account category.

          ledger_id: The id of the ledger that this account category belongs to.

          name: The name of the ledger account category.

          normal_balance: The normal balance of the ledger account category.

          currency_exponent: The currency exponent of the ledger account category.

          description: The description of the ledger account category.

          ledger_account_category_ids: The array of ledger account category ids that this ledger account category
              should be a child of.

          metadata: Additional data represented as key-value pairs. Both the key and value must be
              strings.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        return await self._post(
            "/api/ledger_account_categories",
            body=await async_maybe_transform(
                {
                    "currency": currency,
                    "ledger_id": ledger_id,
                    "name": name,
                    "normal_balance": normal_balance,
                    "currency_exponent": currency_exponent,
                    "description": description,
                    "ledger_account_category_ids": ledger_account_category_ids,
                    "metadata": metadata,
                },
                ledger_account_category_create_params.LedgerAccountCategoryCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=LedgerAccountCategory,
        )

    async def retrieve(
        self,
        id: str,
        *,
        balances: ledger_account_category_retrieve_params.Balances | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> LedgerAccountCategory:
        """
        Get the details on a single ledger account category.

        Args:
          balances: For example, if you want the balances as of a particular time (ISO8601), the
              encoded query string would be `balances%5Beffective_at%5D=2000-12-31T12:00:00Z`.
              The balances as of a time are inclusive of entries with that exact time.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/api/ledger_account_categories/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"balances": balances}, ledger_account_category_retrieve_params.LedgerAccountCategoryRetrieveParams
                ),
            ),
            cast_to=LedgerAccountCategory,
        )

    async def update(
        self,
        id: str,
        *,
        description: Optional[str] | NotGiven = NOT_GIVEN,
        metadata: Dict[str, str] | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> LedgerAccountCategory:
        """
        Update the details of a ledger account category.

        Args:
          description: The description of the ledger account category.

          metadata: Additional data represented as key-value pairs. Both the key and value must be
              strings.

          name: The name of the ledger account category.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/api/ledger_account_categories/{id}",
            body=await async_maybe_transform(
                {
                    "description": description,
                    "metadata": metadata,
                    "name": name,
                },
                ledger_account_category_update_params.LedgerAccountCategoryUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=LedgerAccountCategory,
        )

    def list(
        self,
        *,
        id: List[str] | NotGiven = NOT_GIVEN,
        after_cursor: Optional[str] | NotGiven = NOT_GIVEN,
        balances: ledger_account_category_list_params.Balances | NotGiven = NOT_GIVEN,
        currency: str | NotGiven = NOT_GIVEN,
        ledger_account_id: str | NotGiven = NOT_GIVEN,
        ledger_id: str | NotGiven = NOT_GIVEN,
        metadata: Dict[str, str] | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        parent_ledger_account_category_id: str | NotGiven = NOT_GIVEN,
        per_page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[LedgerAccountCategory, AsyncPage[LedgerAccountCategory]]:
        """
        Get a list of ledger account categories.

        Args:
          id: If you have specific IDs to retrieve in bulk, you can pass them as query
              parameters delimited with `id[]=`, for example `?id[]=123&id[]=abc`.

          balances: For example, if you want the balances as of a particular time (ISO8601), the
              encoded query string would be `balances%5Beffective_at%5D=2000-12-31T12:00:00Z`.
              The balances as of a time are inclusive of entries with that exact time.

          ledger_account_id: Query categories which contain a ledger account directly or through child
              categories.

          metadata: For example, if you want to query for records with metadata key `Type` and value
              `Loan`, the query would be `metadata%5BType%5D=Loan`. This encodes the query
              parameters.

          parent_ledger_account_category_id: Query categories that are nested underneath a parent category

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/ledger_account_categories",
            page=AsyncPage[LedgerAccountCategory],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "after_cursor": after_cursor,
                        "balances": balances,
                        "currency": currency,
                        "ledger_account_id": ledger_account_id,
                        "ledger_id": ledger_id,
                        "metadata": metadata,
                        "name": name,
                        "parent_ledger_account_category_id": parent_ledger_account_category_id,
                        "per_page": per_page,
                    },
                    ledger_account_category_list_params.LedgerAccountCategoryListParams,
                ),
            ),
            model=LedgerAccountCategory,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> LedgerAccountCategory:
        """
        Delete a ledger account category.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/api/ledger_account_categories/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=LedgerAccountCategory,
        )

    async def add_ledger_account(
        self,
        ledger_account_id: str,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> None:
        """
        Add a ledger account to a ledger account category.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not ledger_account_id:
            raise ValueError(f"Expected a non-empty value for `ledger_account_id` but received {ledger_account_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._put(
            f"/api/ledger_account_categories/{id}/ledger_accounts/{ledger_account_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=NoneType,
        )

    async def add_nested_category(
        self,
        sub_category_id: str,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> None:
        """
        Add a ledger account category to a ledger account category.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not sub_category_id:
            raise ValueError(f"Expected a non-empty value for `sub_category_id` but received {sub_category_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._put(
            f"/api/ledger_account_categories/{id}/ledger_account_categories/{sub_category_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=NoneType,
        )

    async def remove_ledger_account(
        self,
        ledger_account_id: str,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> None:
        """
        Remove a ledger account from a ledger account category.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not ledger_account_id:
            raise ValueError(f"Expected a non-empty value for `ledger_account_id` but received {ledger_account_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/api/ledger_account_categories/{id}/ledger_accounts/{ledger_account_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=NoneType,
        )

    async def remove_nested_category(
        self,
        sub_category_id: str,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> None:
        """
        Delete a ledger account category from a ledger account category.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not sub_category_id:
            raise ValueError(f"Expected a non-empty value for `sub_category_id` but received {sub_category_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/api/ledger_account_categories/{id}/ledger_account_categories/{sub_category_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=NoneType,
        )


class LedgerAccountCategoriesWithRawResponse:
    def __init__(self, ledger_account_categories: LedgerAccountCategories) -> None:
        self._ledger_account_categories = ledger_account_categories

        self.create = _legacy_response.to_raw_response_wrapper(
            ledger_account_categories.create,
        )
        self.retrieve = _legacy_response.to_raw_response_wrapper(
            ledger_account_categories.retrieve,
        )
        self.update = _legacy_response.to_raw_response_wrapper(
            ledger_account_categories.update,
        )
        self.list = _legacy_response.to_raw_response_wrapper(
            ledger_account_categories.list,
        )
        self.delete = _legacy_response.to_raw_response_wrapper(
            ledger_account_categories.delete,
        )
        self.add_ledger_account = _legacy_response.to_raw_response_wrapper(
            ledger_account_categories.add_ledger_account,
        )
        self.add_nested_category = _legacy_response.to_raw_response_wrapper(
            ledger_account_categories.add_nested_category,
        )
        self.remove_ledger_account = _legacy_response.to_raw_response_wrapper(
            ledger_account_categories.remove_ledger_account,
        )
        self.remove_nested_category = _legacy_response.to_raw_response_wrapper(
            ledger_account_categories.remove_nested_category,
        )


class AsyncLedgerAccountCategoriesWithRawResponse:
    def __init__(self, ledger_account_categories: AsyncLedgerAccountCategories) -> None:
        self._ledger_account_categories = ledger_account_categories

        self.create = _legacy_response.async_to_raw_response_wrapper(
            ledger_account_categories.create,
        )
        self.retrieve = _legacy_response.async_to_raw_response_wrapper(
            ledger_account_categories.retrieve,
        )
        self.update = _legacy_response.async_to_raw_response_wrapper(
            ledger_account_categories.update,
        )
        self.list = _legacy_response.async_to_raw_response_wrapper(
            ledger_account_categories.list,
        )
        self.delete = _legacy_response.async_to_raw_response_wrapper(
            ledger_account_categories.delete,
        )
        self.add_ledger_account = _legacy_response.async_to_raw_response_wrapper(
            ledger_account_categories.add_ledger_account,
        )
        self.add_nested_category = _legacy_response.async_to_raw_response_wrapper(
            ledger_account_categories.add_nested_category,
        )
        self.remove_ledger_account = _legacy_response.async_to_raw_response_wrapper(
            ledger_account_categories.remove_ledger_account,
        )
        self.remove_nested_category = _legacy_response.async_to_raw_response_wrapper(
            ledger_account_categories.remove_nested_category,
        )


class LedgerAccountCategoriesWithStreamingResponse:
    def __init__(self, ledger_account_categories: LedgerAccountCategories) -> None:
        self._ledger_account_categories = ledger_account_categories

        self.create = to_streamed_response_wrapper(
            ledger_account_categories.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            ledger_account_categories.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            ledger_account_categories.update,
        )
        self.list = to_streamed_response_wrapper(
            ledger_account_categories.list,
        )
        self.delete = to_streamed_response_wrapper(
            ledger_account_categories.delete,
        )
        self.add_ledger_account = to_streamed_response_wrapper(
            ledger_account_categories.add_ledger_account,
        )
        self.add_nested_category = to_streamed_response_wrapper(
            ledger_account_categories.add_nested_category,
        )
        self.remove_ledger_account = to_streamed_response_wrapper(
            ledger_account_categories.remove_ledger_account,
        )
        self.remove_nested_category = to_streamed_response_wrapper(
            ledger_account_categories.remove_nested_category,
        )


class AsyncLedgerAccountCategoriesWithStreamingResponse:
    def __init__(self, ledger_account_categories: AsyncLedgerAccountCategories) -> None:
        self._ledger_account_categories = ledger_account_categories

        self.create = async_to_streamed_response_wrapper(
            ledger_account_categories.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            ledger_account_categories.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            ledger_account_categories.update,
        )
        self.list = async_to_streamed_response_wrapper(
            ledger_account_categories.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            ledger_account_categories.delete,
        )
        self.add_ledger_account = async_to_streamed_response_wrapper(
            ledger_account_categories.add_ledger_account,
        )
        self.add_nested_category = async_to_streamed_response_wrapper(
            ledger_account_categories.add_nested_category,
        )
        self.remove_ledger_account = async_to_streamed_response_wrapper(
            ledger_account_categories.remove_ledger_account,
        )
        self.remove_nested_category = async_to_streamed_response_wrapper(
            ledger_account_categories.remove_nested_category,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from modern_treasury import ModernTreasury, AsyncModernTreasury
from modern_treasury.types import Document
from modern_treasury.pagination import SyncPage, AsyncPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestDocuments:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: ModernTreasury) -> None:
        document = client.documents.create(
            documentable_id="documentable_id",
            documentable_type="cases",
            file=b"raw file contents",
        )
        assert_matches_type(Document, document, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: ModernTreasury) -> None:
        document = client.documents.create(
            documentable_id="documentable_id",
            documentable_type="cases",
            file=b"raw file contents",
            document_type="document_type",
        )
        assert_matches_type(Document, document, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: ModernTreasury) -> None:
        response = client.documents.with_raw_response.create(
            documentable_id="documentable_id",
            documentable_type="cases",
            file=b"raw file contents",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = response.parse()
        assert_matches_type(Document, document, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: ModernTreasury) -> None:
        with client.documents.with_streaming_response.create(
            documentable_id="documentable_id",
            documentable_type="cases",
            file=b"raw file contents",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = response.parse()
            assert_matches_type(Document, document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: ModernTreasury) -> None:
        document = client.documents.retrieve(
            "id",
        )
        assert_matches_type(Document, document, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: ModernTreasury) -> None:
        response = client.documents.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = response.parse()
        assert_matches_type(Document, document, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: ModernTreasury) -> None:
        with client.documents.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = response.parse()
            assert_matches_type(Document, document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: ModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.documents.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_list(self, client: ModernTreasury) -> None:
        document = client.documents.list()
        assert_matches_type(SyncPage[Document], document, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: ModernTreasury) -> None:
        document = client.documents.list(
            after_cursor="after_cursor",
            documentable_id="documentable_id",
            documentable_type="cases",
            per_page=0,
        )
        assert_matches_type(SyncPage[Document], document, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: ModernTreasury) -> None:
        response = client.documents.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = response.parse()
        assert_matches_type(SyncPage[Document], document, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: ModernTreasury) -> None:
        with client.documents.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = response.parse()
            assert_matches_type(SyncPage[Document], document, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncDocuments:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncModernTreasury) -> None:
        document = await async_client.documents.create(
            documentable_id="documentable_id",
            documentable_type="cases",
            file=b"raw file contents",
        )
        assert_matches_type(Document, document, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncModernTreasury) -> None:
        document = await async_client.documents.create(
            documentable_id="documentable_id",
            documentable_type="cases",
            file=b"raw file contents",
            document_type="document_type",
        )
        assert_matches_type(Document, document, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.documents.with_raw_response.create(
            documentable_id="documentable_id",
            documentable_type="cases",
            file=b"raw file contents",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = response.parse()
        assert_matches_type(Document, document, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.documents.with_streaming_response.create(
            documentable_id="documentable_id",
            documentable_type="cases",
            file=b"raw file contents",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = await response.parse()
            assert_matches_type(Document, document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncModernTreasury) -> None:
        document = await async_client.documents.retrieve(
            "id",
        )
        assert_matches_type(Document, document, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.documents.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = response.parse()
        assert_matches_type(Document, document, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.documents.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = await response.parse()
            assert_matches_type(Document, document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.documents.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncModernTreasury) -> None:
        document = await async_client.documents.list()
        assert_matches_type(AsyncPage[Document], document, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncModernTreasury) -> None:
        document = await async_client.documents.list(
            after_cursor="after_cursor",
            documentable_id="documentable_id",
            documentable_type="cases",
            per_page=0,
        )
        assert_matches_type(AsyncPage[Document], document, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.documents.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = response.parse()
        assert_matches_type(AsyncPage[Document], document, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.documents.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = await response.parse()
            assert_matches_type(AsyncPage[Document], document, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Optional
from datetime import date
from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["InvoiceListParams"]


class InvoiceListParams(TypedDict, total=False):
    after_cursor: Optional[str]

    counterparty_id: str

    due_date_end: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """An inclusive upper bound for searching due_date"""

    due_date_start: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """An inclusive lower bound for searching due_date"""

    expected_payment_id: str

    metadata: Dict[str, str]
    """
    For example, if you want to query for records with metadata key `Type` and value
    `Loan`, the query would be `metadata%5BType%5D=Loan`. This encodes the query
    parameters.
    """

    number: str
    """A unique record number assigned to each invoice that is issued."""

    originating_account_id: str

    payment_order_id: str

    per_page: int

    status: Literal["draft", "paid", "partially_paid", "payment_pending", "unpaid", "voided"]



#!/usr/bin/env poetry run python

from modern_treasury import ModernTreasury

mt = ModernTreasury()

account = mt.external_accounts.create(
    name="Test",
    counterparty_id="123",
    party_address={"country": "USA", "locality": "New York City"},
)

print(account.party_address)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, TypedDict

__all__ = ["ConnectionLegalEntityListParams"]


class ConnectionLegalEntityListParams(TypedDict, total=False):
    after_cursor: Optional[str]

    connection_id: str

    legal_entity_id: str

    per_page: int

    status: Literal["completed", "denied", "failed", "processing"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Iterable, Optional
from datetime import date
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo
from .bank_settings_param import BankSettingsParam
from .wealth_and_employment_details_param import WealthAndEmploymentDetailsParam

__all__ = [
    "ConnectionLegalEntityCreateParams",
    "LegalEntity",
    "LegalEntityAddresses",
    "LegalEntityAddress",
    "LegalEntityIdentifications",
    "LegalEntityIdentification",
    "LegalEntityLegalEntityAssociations",
    "LegalEntityLegalEntityAssociation",
    "LegalEntityLegalEntityAssociationsChildLegalEntity",
    "LegalEntityLegalEntityAssociationChildLegalEntity",
    "LegalEntityLegalEntityAssociationsChildLegalEntityAddresses",
    "LegalEntityLegalEntityAssociationChildLegalEntityAddress",
    "LegalEntityLegalEntityAssociationsChildLegalEntityIdentifications",
    "LegalEntityLegalEntityAssociationChildLegalEntityIdentification",
    "LegalEntityLegalEntityAssociationsChildLegalEntityPhoneNumbers",
    "LegalEntityLegalEntityAssociationChildLegalEntityPhoneNumber",
    "LegalEntityPhoneNumbers",
    "LegalEntityPhoneNumber",
]


class ConnectionLegalEntityCreateParams(TypedDict, total=False):
    connection_id: Required[str]
    """The ID of the connection."""

    legal_entity: LegalEntity
    """The legal entity."""

    legal_entity_id: str
    """The ID of the legal entity."""


class LegalEntityAddress(TypedDict, total=False):
    country: Required[Optional[str]]
    """Country code conforms to [ISO 3166-1 alpha-2]"""

    line1: Required[Optional[str]]

    locality: Required[Optional[str]]
    """Locality or City."""

    postal_code: Required[Optional[str]]
    """The postal code of the address."""

    region: Required[Optional[str]]
    """Region or State."""

    address_types: List[Literal["business", "mailing", "other", "po_box", "residential"]]
    """The types of this address."""

    line2: Optional[str]


LegalEntityAddresses = LegalEntityAddress
"""This type is deprecated and will be removed in a future release.

Please use LegalEntityAddress instead.
"""


class LegalEntityIdentification(TypedDict, total=False):
    id_number: Required[str]
    """The ID number of identification document."""

    id_type: Required[
        Literal[
            "ar_cuil",
            "ar_cuit",
            "br_cnpj",
            "br_cpf",
            "cl_run",
            "cl_rut",
            "co_cedulas",
            "co_nit",
            "hn_id",
            "hn_rtn",
            "in_lei",
            "kr_brn",
            "kr_crn",
            "kr_rrn",
            "passport",
            "sa_tin",
            "sa_vat",
            "us_ein",
            "us_itin",
            "us_ssn",
            "vn_tin",
        ]
    ]
    """The type of ID number."""

    issuing_country: Optional[str]
    """
    The ISO 3166-1 alpha-2 country code of the country that issued the
    identification
    """


LegalEntityIdentifications = LegalEntityIdentification
"""This type is deprecated and will be removed in a future release.

Please use LegalEntityIdentification instead.
"""


class LegalEntityLegalEntityAssociationChildLegalEntityAddress(TypedDict, total=False):
    country: Required[Optional[str]]
    """Country code conforms to [ISO 3166-1 alpha-2]"""

    line1: Required[Optional[str]]

    locality: Required[Optional[str]]
    """Locality or City."""

    postal_code: Required[Optional[str]]
    """The postal code of the address."""

    region: Required[Optional[str]]
    """Region or State."""

    address_types: List[Literal["business", "mailing", "other", "po_box", "residential"]]
    """The types of this address."""

    line2: Optional[str]


LegalEntityLegalEntityAssociationsChildLegalEntityAddresses = LegalEntityLegalEntityAssociationChildLegalEntityAddress
"""This type is deprecated and will be removed in a future release.

Please use LegalEntityLegalEntityAssociationChildLegalEntityAddress instead.
"""


class LegalEntityLegalEntityAssociationChildLegalEntityIdentification(TypedDict, total=False):
    id_number: Required[str]
    """The ID number of identification document."""

    id_type: Required[
        Literal[
            "ar_cuil",
            "ar_cuit",
            "br_cnpj",
            "br_cpf",
            "cl_run",
            "cl_rut",
            "co_cedulas",
            "co_nit",
            "hn_id",
            "hn_rtn",
            "in_lei",
            "kr_brn",
            "kr_crn",
            "kr_rrn",
            "passport",
            "sa_tin",
            "sa_vat",
            "us_ein",
            "us_itin",
            "us_ssn",
            "vn_tin",
        ]
    ]
    """The type of ID number."""

    issuing_country: Optional[str]
    """
    The ISO 3166-1 alpha-2 country code of the country that issued the
    identification
    """


LegalEntityLegalEntityAssociationsChildLegalEntityIdentifications = (
    LegalEntityLegalEntityAssociationChildLegalEntityIdentification
)
"""This type is deprecated and will be removed in a future release.

Please use LegalEntityLegalEntityAssociationChildLegalEntityIdentification instead.
"""


class LegalEntityLegalEntityAssociationChildLegalEntityPhoneNumber(TypedDict, total=False):
    phone_number: str


LegalEntityLegalEntityAssociationsChildLegalEntityPhoneNumbers = (
    LegalEntityLegalEntityAssociationChildLegalEntityPhoneNumber
)
"""This type is deprecated and will be removed in a future release.

Please use LegalEntityLegalEntityAssociationChildLegalEntityPhoneNumber instead.
"""


class LegalEntityLegalEntityAssociationChildLegalEntity(TypedDict, total=False):
    addresses: Iterable[LegalEntityLegalEntityAssociationChildLegalEntityAddress]
    """A list of addresses for the entity."""

    bank_settings: Optional[BankSettingsParam]

    business_name: Optional[str]
    """The business's legal business name."""

    citizenship_country: Optional[str]
    """The country of citizenship for an individual."""

    date_formed: Annotated[Union[str, date, None], PropertyInfo(format="iso8601")]
    """A business's formation date (YYYY-MM-DD)."""

    date_of_birth: Annotated[Union[str, date, None], PropertyInfo(format="iso8601")]
    """An individual's date of birth (YYYY-MM-DD)."""

    doing_business_as_names: List[str]

    email: Optional[str]
    """The entity's primary email."""

    first_name: Optional[str]
    """An individual's first name."""

    identifications: Iterable[LegalEntityLegalEntityAssociationChildLegalEntityIdentification]
    """A list of identifications for the legal entity."""

    last_name: Optional[str]
    """An individual's last name."""

    legal_entity_type: Literal["business", "individual"]
    """The type of legal entity."""

    legal_structure: Optional[
        Literal["corporation", "llc", "non_profit", "partnership", "sole_proprietorship", "trust"]
    ]
    """The business's legal structure."""

    metadata: Dict[str, str]
    """Additional data represented as key-value pairs.

    Both the key and value must be strings.
    """

    middle_name: Optional[str]
    """An individual's middle name."""

    phone_numbers: Iterable[LegalEntityLegalEntityAssociationChildLegalEntityPhoneNumber]

    politically_exposed_person: Optional[bool]
    """Whether the individual is a politically exposed person."""

    preferred_name: Optional[str]
    """An individual's preferred name."""

    prefix: Optional[str]
    """An individual's prefix."""

    risk_rating: Optional[Literal["low", "medium", "high"]]
    """The risk rating of the legal entity. One of low, medium, high."""

    suffix: Optional[str]
    """An individual's suffix."""

    wealth_and_employment_details: Optional[WealthAndEmploymentDetailsParam]

    website: Optional[str]
    """The entity's primary website URL."""


LegalEntityLegalEntityAssociationsChildLegalEntity = LegalEntityLegalEntityAssociationChildLegalEntity
"""This type is deprecated and will be removed in a future release.

Please use LegalEntityLegalEntityAssociationChildLegalEntity instead.
"""


class LegalEntityLegalEntityAssociation(TypedDict, total=False):
    relationship_types: Required[List[Literal["beneficial_owner", "control_person"]]]

    child_legal_entity: LegalEntityLegalEntityAssociationChildLegalEntity
    """The child legal entity."""

    child_legal_entity_id: str
    """The ID of the child legal entity."""

    ownership_percentage: Optional[int]
    """The child entity's ownership percentage iff they are a beneficial owner."""

    title: Optional[str]
    """The job title of the child entity at the parent entity."""


LegalEntityLegalEntityAssociations = LegalEntityLegalEntityAssociation
"""This type is deprecated and will be removed in a future release.

Please use LegalEntityLegalEntityAssociation instead.
"""


class LegalEntityPhoneNumber(TypedDict, total=False):
    phone_number: str


LegalEntityPhoneNumbers = LegalEntityPhoneNumber
"""This type is deprecated and will be removed in a future release.

Please use LegalEntityPhoneNumber instead.
"""


class LegalEntity(TypedDict, total=False):
    addresses: Iterable[LegalEntityAddress]
    """A list of addresses for the entity."""

    bank_settings: Optional[BankSettingsParam]

    business_name: Optional[str]
    """The business's legal business name."""

    citizenship_country: Optional[str]
    """The country of citizenship for an individual."""

    date_formed: Annotated[Union[str, date, None], PropertyInfo(format="iso8601")]
    """A business's formation date (YYYY-MM-DD)."""

    date_of_birth: Annotated[Union[str, date, None], PropertyInfo(format="iso8601")]
    """An individual's date of birth (YYYY-MM-DD)."""

    doing_business_as_names: List[str]

    email: Optional[str]
    """The entity's primary email."""

    first_name: Optional[str]
    """An individual's first name."""

    identifications: Iterable[LegalEntityIdentification]
    """A list of identifications for the legal entity."""

    last_name: Optional[str]
    """An individual's last name."""

    legal_entity_associations: Optional[Iterable[LegalEntityLegalEntityAssociation]]
    """The legal entity associations and its child legal entities."""

    legal_entity_type: Literal["business", "individual"]
    """The type of legal entity."""

    legal_structure: Optional[
        Literal["corporation", "llc", "non_profit", "partnership", "sole_proprietorship", "trust"]
    ]
    """The business's legal structure."""

    metadata: Dict[str, str]
    """Additional data represented as key-value pairs.

    Both the key and value must be strings.
    """

    middle_name: Optional[str]
    """An individual's middle name."""

    phone_numbers: Iterable[LegalEntityPhoneNumber]

    politically_exposed_person: Optional[bool]
    """Whether the individual is a politically exposed person."""

    preferred_name: Optional[str]
    """An individual's preferred name."""

    prefix: Optional[str]
    """An individual's prefix."""

    risk_rating: Optional[Literal["low", "medium", "high"]]
    """The risk rating of the legal entity. One of low, medium, high."""

    suffix: Optional[str]
    """An individual's suffix."""

    wealth_and_employment_details: Optional[WealthAndEmploymentDetailsParam]

    website: Optional[str]
    """The entity's primary website URL."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import hmac
from hashlib import sha256

from .._types import (
    HeadersLike,
)
from .._utils import (
    get_required_header,
)
from .._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["Webhooks", "AsyncWebhooks"]


class Webhooks(SyncAPIResource):
    def get_signature(
        self,
        payload: str,
        *,
        key: str | None = None,
    ) -> str:
        """
        To verify that a webhook was actually sent by Modern Treasury, every payload is
        signed with a signature that is passed through the `X-Signature` HTTP header.

        This method will generate a signature based off of your webhook key which can be
        found in the Developer Settings,
        https://app.moderntreasury.com/developers/webhooks, and the webhook payload.

        You can then compare the generated signature with the signature sent with the
        request, if they match then the webhook was sent by Modern Treasury.
        """
        if key is None:
            key = self._client.webhook_key

        if key is None:
            raise ValueError(
                "The webhook key must either be set using the env var, MODERN_TREASURY_WEBHOOK_KEY, on the client class, ModernTreasury(webhook_key='123'), or passed to this function"
            )

        if not key:
            raise ValueError("The webhook is set but appears to be empty")

        mac = hmac.new(
            key.encode("utf-8"),
            msg=payload.encode("utf-8"),
            digestmod=sha256,
        )
        return mac.hexdigest()

    def validate_signature(
        self,
        payload: str,
        headers: HeadersLike,
        *,
        key: str | None = None,
    ) -> bool:
        """Returns whether or not the webhook payload was sent by Modern Treasury."""
        signature = self.get_signature(payload, key=key)
        expected_signature = get_required_header(headers, "X-Signature")

        return signature == expected_signature


class AsyncWebhooks(AsyncAPIResource):
    def get_signature(
        self,
        payload: str,
        *,
        key: str | None = None,
    ) -> str:
        """
        To verify that a webhook was actually sent by Modern Treasury, every payload is
        signed with a signature that is passed through the `X-Signature` HTTP header.

        This method will generate a signature based off of your webhook key which can be
        found in the Developer Settings,
        https://app.moderntreasury.com/developers/webhooks, and the webhook payload.

        You can then compare the generated signature with the signature sent with the
        request, if they match then the webhook was sent by Modern Treasury.
        """
        if key is None:
            key = self._client.webhook_key

        if key is None:
            raise ValueError(
                "The webhook key must either be set using the env var, MODERN_TREASURY_WEBHOOK_KEY, on the client class, ModernTreasury(webhook_key='123'), or passed to this function"
            )

        if not key:
            raise ValueError("The webhook is set but appears to be empty")

        mac = hmac.new(
            key.encode("utf-8"),
            msg=payload.encode("utf-8"),
            digestmod=sha256,
        )
        return mac.hexdigest()

    def validate_signature(
        self,
        payload: str,
        headers: HeadersLike,
        *,
        key: str | None = None,
    ) -> bool:
        """Returns whether or not the webhook payload was sent by Modern Treasury."""
        signature = self.get_signature(payload, key=key)
        expected_signature = get_required_header(headers, "X-Signature")

        return signature == expected_signature



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import TypedDict

__all__ = ["IncomingPaymentDetailUpdateParams"]


class IncomingPaymentDetailUpdateParams(TypedDict, total=False):
    metadata: Dict[str, str]
    """Additional data in the form of key-value pairs.

    Pairs can be removed by passing an empty string or `null` as the value.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["ConnectionLegalEntityUpdateParams"]


class ConnectionLegalEntityUpdateParams(TypedDict, total=False):
    status: Literal["processing"]
    """The status of the connection legal entity."""
